@@ -38,8 +38,6 @@ else:
 
 
 BASE_DIR = os.path.join(tempfile.gettempdir(), "mss{}".format(SHA))
-VT_CACHE = os.path.join(BASE_DIR, 'vt_cache')
 DATA_DIR = os.path.join(BASE_DIR, 'testdata')
 SERVER_CONFIG_FILE = os.path.join(BASE_DIR, "mss_wms_settings.py")
-VALID_TIME_CACHE = os.path.join(BASE_DIR, 'vt_cache')
 os.environ["MSS_CONFIG_PATH"] = BASE_DIR
