@@ -36,19 +36,20 @@
 
 from __future__ import division
 
-from past.builtins import basestring
 from builtins import str
 
 import datetime
 import codecs
 import logging
+import os
 import xml.dom.minidom
+import xml.parsers.expat
 
 # related third party imports
-from mslib.msui.mss_qt import QtGui, QtCore, QtWidgets, QString, localized_float, USE_PYQT5
+from mslib.msui.mss_qt import QtGui, QtCore, QtWidgets, QString, variant_to_string, variant_to_float, USE_PYQT5
 
 # local application imports
-from mslib import utils
+from mslib import utils, __version__
 from mslib import thermolib
 from mslib.utils import config_loader, find_location, save_settings_pickle, load_settings_pickle
 from mslib.msui.performance_settings import DEFAULT_PERFORMANCE
@@ -108,7 +109,7 @@ class Waypoint(object):
         self.pressure = thermolib.flightlevel2pressure(flightlevel)
         self.distance_to_prev = 0.
         self.distance_total = 0.
-        self._comments = comments
+        self.comments = comments
 
         # Performance fields (for values read from the flight performance
         # service).
@@ -122,23 +123,12 @@ class Waypoint(object):
         self.wpnumber_major = None
         self.wpnumber_minor = None
 
-    def get_comments(self):
-        return self._comments
-
-    def set_comments(self, string):
-        if isinstance(string, basestring):
-            self._comments = QString(string)
-        else:
-            self._comments = string
-
-    comments = property(get_comments, set_comments)
-
     def __str__(self):
         """String representation of the waypoint (e.g., when used with the print
            statement).
         """
-        return "WAYPOINT(LAT={:f}, LON={:f}, FL={:f})".format(self.lat, self.lon,
-                                                              self.flightlevel)
+        return "WAYPOINT(LAT={:f}, LON={:f}, FL={:f})".format(
+            self.lat, self.lon, self.flightlevel)
 
 
 class WaypointsTableModel(QtCore.QAbstractTableModel):
@@ -162,24 +152,24 @@ class WaypointsTableModel(QtCore.QAbstractTableModel):
 
         # self.aircraft.setErrorHandling("permissive")
         self.settings_tag = "performance"
-        self.loadSettings()
+        self.load_settings()
 
         # If a filename is passed to the constructor, load data from this file.
         if filename:
             if filename.endswith(".ftml"):
-                self.loadFromFTML(filename)
+                self.load_from_ftml(filename)
             else:
-                logging.debug(u"No known file extension! '{}'".format(filename))
+                logging.debug(u"No known file extension! '%s'", filename)
 
         if waypoints:
-            self.replaceWaypoints(waypoints)
+            self.replace_waypoints(waypoints)
 
-    def loadSettings(self):
+    def load_settings(self):
         """Load settings from the file self.settingsfile.
         """
         self.performance_settings = load_settings_pickle(self.settings_tag, DEFAULT_PERFORMANCE)
 
-    def saveSettings(self):
+    def save_settings(self):
         """Save the current settings (map appearance) to the file
            self.settingsfile.
         """
@@ -187,14 +177,9 @@ class WaypointsTableModel(QtCore.QAbstractTableModel):
         # http://stackoverflow.com/questions/200599/whats-the-best-way-to-store-simple-user-settings-in-python
         save_settings_pickle(self.settings_tag, self.performance_settings)
 
-    def setName(self, name):
-        self.name = name
-        self.modified = True
-
     def flags(self, index):
         """Used to specify which table columns can be edited by the user;
            overrides the corresponding QAbstractTableModel method.
-
         """
         if not index.isValid():
             return QtCore.Qt.ItemIsEnabled
@@ -232,17 +217,17 @@ class WaypointsTableModel(QtCore.QAbstractTableModel):
             return QtCore.QVariant(int(QtCore.Qt.AlignLeft | QtCore.Qt.AlignVCenter))
         return QtCore.QVariant()
 
-    def waypointData(self, row):
+    def waypoint_data(self, row):
         """Get the waypoint object defining the given row.
         """
         return self.waypoints[row]
 
-    def allWaypointData(self):
+    def all_waypoint_data(self):
         """Return the entire list of waypoints.
         """
         return self.waypoints
 
-    def intermediatePoints(self, numpoints=101, connection="greatcircle"):
+    def intermediate_points(self, numpoints=101, connection="greatcircle"):
         """Compute intermediate points between the waypoints.
 
         See mss_util.path_points() for additional arguments.
@@ -288,25 +273,16 @@ class WaypointsTableModel(QtCore.QAbstractTableModel):
         NOTE: Performance computations loose their validity if a change is made.
         """
         if index.isValid() and 0 <= index.row() < len(self.waypoints):
-            if isinstance(value, QtCore.QVariant):
-                if USE_PYQT5:
-                    value = value.value()
-                else:
-                    value_p, isok = value.toDouble()
-                    if isok:
-                        value = value_p
-                    else:
-                        value = value.toString()
             waypoint = self.waypoints[index.row()]
             column = index.column()
             index2 = index  # in most cases only one field is being changed
             if column == LOCATION:
-                waypoint.location = value
+                waypoint.location = variant_to_string(value)
             elif column == LAT:
                 try:
                     # The table fields accept basically any input.
                     # If the string cannot be converted to "float" (raises ValueError), the user input is discarded.
-                    value = localized_float(value)
+                    value = variant_to_float(value)
                 except TypeError as ex:
                     logging.error("unexpected error: %s %s %s %s", type(ex), ex, type(value), value)
                 except ValueError:
@@ -314,7 +290,7 @@ class WaypointsTableModel(QtCore.QAbstractTableModel):
                 else:
                     waypoint.lat = value
                     waypoint.location = u""
-                    loc = find_location(waypoint.lat, waypoint.lon, 0)
+                    loc = find_location(waypoint.lat, waypoint.lon, 1e-3)
                     if loc is not None:
                         waypoint.lat, waypoint.lon = loc[0]
                         waypoint.location = loc[1]
@@ -331,7 +307,7 @@ class WaypointsTableModel(QtCore.QAbstractTableModel):
                 try:
                     # The table fields accept basically any input.
                     # If the string cannot be converted to "float" (raises ValueError), the user input is discarded.
-                    value = localized_float(value)
+                    value = variant_to_float(value)
                 except TypeError as ex:
                     logging.error("unexpected error: %s %s %s %s", type(ex), ex, type(value), value)
                 except ValueError:
@@ -339,7 +315,7 @@ class WaypointsTableModel(QtCore.QAbstractTableModel):
                 else:
                     waypoint.lon = value
                     waypoint.location = u""
-                    loc = find_location(waypoint.lat, waypoint.lon, 0)
+                    loc = find_location(waypoint.lat, waypoint.lon, 1e-3)
                     if loc is not None:
                         waypoint.lat, waypoint.lon = loc[0]
                         waypoint.location = loc[1]
@@ -350,7 +326,7 @@ class WaypointsTableModel(QtCore.QAbstractTableModel):
                 try:
                     # The table fields accept basically any input.
                     # If the string cannot be converted to "float" (raises ValueError), the user input is discarded.
-                    flightlevel = localized_float(value)
+                    flightlevel = variant_to_float(value)
                     pressure = thermolib.flightlevel2pressure(flightlevel)
                 except TypeError as ex:
                     logging.error("unexpected error: %s %s %s %s", type(ex), ex, type(value), value)
@@ -368,7 +344,7 @@ class WaypointsTableModel(QtCore.QAbstractTableModel):
                 try:
                     # The table fields accept basically any input.
                     # If the string cannot be converted to "float" (raises ValueError), the user input is discarded.
-                    pressure = localized_float(value) * 100  # convert hPa to Pa
+                    pressure = variant_to_float(value) * 100  # convert hPa to Pa
                     if pressure > 200000:
                         raise ValueError
                     flightlevel = round(thermolib.pressure2flightlevel(pressure))
@@ -384,7 +360,7 @@ class WaypointsTableModel(QtCore.QAbstractTableModel):
                         self.update_distances(index.row())
                     index2 = self.createIndex(index.row(), FLIGHTLEVEL)
             else:
-                waypoint.comments = value
+                waypoint.comments = variant_to_string(value)
             self.modified = True
             # Performance computations loose their validity if a change is made.
             self.dataChanged.emit(index, index2)
@@ -516,28 +492,27 @@ class WaypointsTableModel(QtCore.QAbstractTableModel):
         index1 = self.createIndex(0, TIME_UTC)
         self.dataChanged.emit(index1, index1)
 
-
-    def invertDirection(self):
+    def invert_direction(self):
         self.waypoints = self.waypoints[::-1]
         if len(self.waypoints) > 0:
             self.waypoints[0].distance_to_prev = 0
             self.waypoints[0].distance_total = 0
         for i in range(1, len(self.waypoints)):
-            wp_comm = str(self.waypoints[i].comments)
+            wp_comm = self.waypoints[i].comments
             if len(wp_comm) == 9 and wp_comm.startswith("Hexagon "):
-                wp_comm = "Hexagon {:d}".format(8 - int(wp_comm[-1]))
-                self.waypoints[i].comments = QString(wp_comm)
+                wp_comm = u"Hexagon {:d}".format(8 - int(wp_comm[-1]))
+                self.waypoints[i].comments = wp_comm
         self.update_distances(position=0, rows=len(self.waypoints))
         index = self.index(0, 0)
 
         self.layoutChanged.emit()
         self.dataChanged.emit(index, index)
 
-    def replaceWaypoints(self, new_waypoints):
+    def replace_waypoints(self, new_waypoints):
         self.waypoints = []
         self.insertRows(0, rows=len(new_waypoints), waypoints=new_waypoints)
 
-    def saveToFTML(self, filename=None):
+    def save_to_ftml(self, filename=None):
         """Save the flight track to an XML file.
 
         Arguments:
@@ -545,21 +520,17 @@ class WaypointsTableModel(QtCore.QAbstractTableModel):
                     specified filename will be used. If no filename has been
                     specified at all, a ValueError exception will be raised.
         """
-        if filename:
-            self.filename = filename
-        if not self.filename:
-            raise ValueError("filename to save flight track cannot be None")
+        if not filename:
+            raise ValueError("filename to save flight track cannot be None or empty")
+
+        self.filename = filename
+        self.name = os.path.basename(filename.replace(".ftml", "").strip())
 
         doc = xml.dom.minidom.Document()
 
         ft_el = doc.createElement("FlightTrack")
+        ft_el.setAttribute("version", str(__version__))
         doc.appendChild(ft_el)
-
-        # Element that contains the name of the flight track.
-        name_el = doc.createElement("Name")
-        name_el.appendChild(doc.createTextNode(self.name))
-        ft_el.appendChild(name_el)
-
         # The list of waypoint elements.
         wp_el = doc.createElement("ListOfWaypoints")
         ft_el.appendChild(wp_el)
@@ -578,15 +549,17 @@ class WaypointsTableModel(QtCore.QAbstractTableModel):
         with codecs.open(self.filename, 'w', encoding="utf-8") as file_object:
             doc.writexml(file_object, indent="  ", addindent="  ", newl="\n", encoding="utf-8")
 
-    def loadFromFTML(self, filename):
+    def load_from_ftml(self, filename):
         """Load a flight track from an XML file at <filename>.
         """
-        doc = xml.dom.minidom.parse(filename)
+        try:
+            doc = xml.dom.minidom.parse(filename)
+        except xml.parsers.expat.ExpatError as ex:
+            raise SyntaxError(str(ex))
 
         ft_el = doc.getElementsByTagName("FlightTrack")[0]
 
-        name_el = ft_el.getElementsByTagName("Name")[0]
-        self.name = name_el.childNodes[0].data.strip()
+        self.name = os.path.basename(filename.replace(".ftml", "").strip())
 
         waypoints_list = []
         for wp_el in ft_el.getElementsByTagName("Waypoint"):
@@ -605,9 +578,9 @@ class WaypointsTableModel(QtCore.QAbstractTableModel):
             waypoints_list.append(Waypoint(lat, lon, flightlevel,
                                            location=location,
                                            comments=comments))
-        self.replaceWaypoints(waypoints_list)
+        self.replace_waypoints(waypoints_list)
 
-    def getFilename(self):
+    def get_filename(self):
         return self.filename
 
 
@@ -629,7 +602,7 @@ class WaypointDelegate(QtWidgets.QItemDelegate):
         """Colours waypoints with a minor waypoint number (i.e. intermediate
            waypoints generated by the flight performance service) in red.
         """
-        wpnumber_minor = index.model().waypointData(index.row()).wpnumber_minor
+        wpnumber_minor = index.model().waypoint_data(index.row()).wpnumber_minor
         if wpnumber_minor is not None and wpnumber_minor > 0:
             newpalette = QtGui.QPalette(option.palette)
             colour = QtGui.QColor(170, 0, 0)  # dark red
@@ -648,7 +621,7 @@ class WaypointDelegate(QtWidgets.QItemDelegate):
             locations = config_loader(dataset='locations', default=mss_default.locations)
             adds = list(locations.keys())
             if self.parent() is not None:
-                for loc in [wp.location for wp in self.parent().waypoints_model.allWaypointData() if
+                for loc in [wp.location for wp in self.parent().waypoints_model.all_waypoint_data() if
                             wp.location != ""]:
                     if loc not in adds:
                         adds.append(loc)
@@ -688,7 +661,7 @@ class WaypointDelegate(QtWidgets.QItemDelegate):
                               update=False)
                 model.setData(index.sibling(index.row(), LON), QtCore.QVariant(lon))
             else:
-                for wp in self.parent().waypoints_model.allWaypointData():
+                for wp in self.parent().waypoints_model.all_waypoint_data():
                     if loc == wp.location:
                         lat, lon = wp.lat, wp.lon
                         # Don't update distances and flight performance twice, hence
