@@ -32,19 +32,19 @@ from __future__ import print_function
 
 from builtins import str
 
+import argparse
 import copy
+import functools
+import hashlib
 import importlib
 import logging
 import os
+import platform
 import re
+import requests
 import shutil
 import sys
 import types
-import functools
-import platform
-import argparse
-import hashlib
-import requests
 
 from mslib import __version__
 from mslib.msui.mss_qt import ui_mainwindow as ui
@@ -55,8 +55,8 @@ from mslib.msui import topview
 from mslib.msui import sideview
 from mslib.msui import timeseriesview
 from mslib.msui import trajectories_tool
-from mslib.msui import loopview
 from mslib.msui import constants
+from mslib.msui import wms_control
 from mslib.utils import config_loader, setup_logging
 from mslib.msui import MissionSupportSystemDefaultConfig as mss_default
 from mslib.plugins.io.csv import load_from_csv, save_to_csv
@@ -139,14 +139,6 @@ class QFlightTrackListWidgetItem(QtWidgets.QListWidgetItem):
         self.parent = parent
         self.flighttrack_model = flighttrack_model
 
-        if parent is not None:
-            parent.itemChanged.connect(self.nameChanged)
-
-    def nameChanged(self, item):
-        """Slot to change the name of a flight track.
-        """
-        item.flighttrack_model.setName(str(item.text()))
-
 
 class MSS_AboutDialog(QtWidgets.QDialog, ui_ab.Ui_AboutMSUIDialog):
     """Dialog showing information about MSUI. Most of the displayed text is
@@ -188,53 +180,91 @@ class MSSMainWindow(QtWidgets.QMainWindow, ui.Ui_MSSMainWindow):
         # Reference to the flight track that is currently displayed in the
         # views.
         self.active_flight_track = None
-        self.lastSaveDir = os.getcwd()
+        self.last_save_directory = os.getcwd()
 
         # Connect Qt SIGNALs:
         # ===================
 
         # File menu.
-        self.actionNewFlightTrack.triggered.connect(functools.partial(self.createNewFlightTrack, None, None, False))
-        self.actionOpenFlightTrack.triggered.connect(self.openFlightTrack)
-        self.actionCloseSelectedFlightTrack.triggered.connect(self.closeFlightTrack)
-        self.actionSaveActiveFlightTrack.triggered.connect(self.saveFlightTrack)
-        self.actionSaveActiveFlightTrackAs.triggered.connect(self.saveFlightTrackAs)
+        self.actionNewFlightTrack.triggered.connect(functools.partial(self.create_new_flight_track, None, None))
+        self.actionOpenFlightTrack.triggered.connect(self.open_flight_track)
+        self.actionActivateSelectedFlightTrack.triggered.connect(self.activate_selected_flight_track)
+        self.actionCloseSelectedFlightTrack.triggered.connect(self.close_selected_flight_track)
+        self.actionSaveActiveFlightTrack.triggered.connect(self.save_flight_track)
+        self.actionSaveActiveFlightTrackAs.triggered.connect(self.save_flight_track_as)
 
         # Views menu.
-        self.actionTopView.triggered.connect(self.createNewView)
-        self.actionSideView.triggered.connect(self.createNewView)
-        self.actionTableView.triggered.connect(self.createNewView)
+        self.actionTopView.triggered.connect(self.create_new_view)
+        self.actionSideView.triggered.connect(self.create_new_view)
+        self.actionTableView.triggered.connect(self.create_new_view)
 
         # Tools menu.
-        self.actionTrajectoryToolLagranto.triggered.connect(self.createNewTool)
-        self.actionTimeSeriesViewTrajectories.triggered.connect(self.createNewTool)
-        self.actionLoopView.triggered.connect(self.createNewTool)
+        self.actionTrajectoryToolLagranto.triggered.connect(self.create_new_tool)
+        self.actionTimeSeriesViewTrajectories.triggered.connect(self.create_new_tool)
 
         # Help menu.
-        self.actionOnlineHelp.triggered.connect(self.showOnlineHelp)
-        self.actionAboutMSUI.triggered.connect(self.showAboutDlg)
+        self.actionOnlineHelp.triggered.connect(self.show_online_help)
+        self.actionAboutMSUI.triggered.connect(self.show_about_dialog)
 
         # Load Config
-        self.actionLoad_Configuration.triggered.connect(self.openConfigFile)
+        self.actionLoad_Configuration.triggered.connect(self.open_config_file)
 
         # Flight Tracks.
-        self.listFlightTracks.itemChanged.connect(self.flightTrackNameChanged)
-        self.btSelectFlightTrack.clicked.connect(self.setFlightTrackActive)
+        self.listFlightTracks.itemActivated.connect(self.activate_flight_track)
 
         # Views.
-        self.listViews.itemActivated.connect(self.activateSubWindow)
+        self.listViews.itemActivated.connect(self.activate_sub_window)
 
         # Tools.
-        self.listTools.itemActivated.connect(self.activateSubWindow)
+        self.listTools.itemActivated.connect(self.activate_sub_window)
 
-        self.addImportFilter("CSV", "csv", load_from_csv)
-        self.addExportFilter("CSV", "csv", save_to_csv)
+        self.add_import_filter("CSV", "csv", load_from_csv)
+        self.add_export_filter("CSV", "csv", save_to_csv)
 
         self._imported_plugins, self._exported_plugins = {}, {}
-        self.addPlugins()
-        # self.actionLoopView.setVisible(config_loader(dataset="enable_loopview", default=False))
+        self.add_plugins()
 
-    def addPlugins(self):
+        preload_urls = config_loader(dataset="WMS_preload", default=[])
+        self.preload_wms(preload_urls)
+
+    @staticmethod
+    def preload_wms(urls):
+        """
+        This method accesses a list of WMS servers and load their capability documents.
+        :param urls: List of URLs
+        """
+        pdlg = QtWidgets.QProgressDialog("Preloading WMS servers...", "Cancel", 0, len(urls))
+        pdlg.reset()
+        pdlg.setValue(0)
+        pdlg.setModal(True)
+        pdlg.show()
+        QtWidgets.QApplication.processEvents()
+        for i, base_url in enumerate(urls):
+            pdlg.setValue(i)
+            QtWidgets.QApplication.processEvents()
+            logging.info(base_url)
+            # initialize login cache fomr config file, but do not overwrite existing keys
+            for key, value in list(config_loader(dataset="WMS_login", default={}).items()):
+                if key not in constants.WMS_LOGIN_CACHE:
+                    constants.WMS_LOGIN_CACHE[key] = value
+            username, password = constants.WMS_LOGIN_CACHE.get(base_url, (None, None))
+
+            try:
+                request = requests.get(base_url)
+                if pdlg.wasCanceled():
+                    break
+                wms = wms_control.MSSWebMapService(request.url, version='1.1.1',
+                                                   username=username, password=password)
+                wms_control.WMS_SERVICE_CACHE[wms.url] = wms
+                logging.info("Stored WMS info for '%s'", wms.url)
+            except Exception as ex:
+                logging.error("Error in preloading '%s': '%s'", type(ex), ex)
+            if pdlg.wasCanceled():
+                break
+        logging.debug("Contents of WMS_SERVICE_CACHE: %s", wms_control.WMS_SERVICE_CACHE.keys())
+        pdlg.close()
+
+    def add_plugins(self):
         self._imported_plugins = config_loader(dataset="import_plugins", default={})
         for name in self._imported_plugins:
             extension, module, function = self._imported_plugins[name]
@@ -249,7 +279,7 @@ class MSSMainWindow(QtWidgets.QMainWindow, ui.Ui_MSSMainWindow):
                         self._imported_plugins, type(ex), ex)))
                 continue
             try:
-                self.addImportFilter(name, extension, getattr(imported_module, function))
+                self.add_import_filter(name, extension, getattr(imported_module, function))
             # wildcard exception to be resilient against error introduced by user code
             except Exception as ex:
                 logging.error(u"Error on installing plugin: %s: %s", type(ex), ex)
@@ -273,7 +303,7 @@ class MSSMainWindow(QtWidgets.QMainWindow, ui.Ui_MSSMainWindow):
                         self._exported_plugins, type(ex), ex)))
                 continue
             try:
-                    self.addExportFilter(name, extension, getattr(imported_module, function))
+                    self.add_export_filter(name, extension, getattr(imported_module, function))
             # wildcard exception to be resilient against error introduced by user code
             except Exception as ex:
                 logging.error(u"Error on installing plugin: %s: %s", type(ex), ex)
@@ -283,7 +313,7 @@ class MSSMainWindow(QtWidgets.QMainWindow, ui.Ui_MSSMainWindow):
                         self._exported_plugins, type(ex), ex)))
                 continue
 
-    def removePlugins(self):
+    def remove_plugins(self):
         for name in self._imported_plugins:
             full_name = "actionImportFlightTrack" + clean_string(name)
             actions = [_x for _x in self.menuImport_Flight_Track.actions()
@@ -300,7 +330,7 @@ class MSSMainWindow(QtWidgets.QMainWindow, ui.Ui_MSSMainWindow):
             self.menuExport_Active_Flight_Track.removeAction(actions[0])
             delattr(self, full_name)
 
-    def addImportFilter(self, name, extension, function):
+    def add_import_filter(self, name, extension, function):
         full_name = "actionImportFlightTrack" + clean_string(name)
         if hasattr(self, full_name):
             raise ValueError(u"'{}' has already been set!".format(full_name))
@@ -312,7 +342,7 @@ class MSSMainWindow(QtWidgets.QMainWindow, ui.Ui_MSSMainWindow):
 
         def load_function_wrapper(self):
             filename = QtWidgets.QFileDialog.getOpenFileName(
-                self, "Import Flight Track", "", name + " (*." + extension + ")")
+                self, "Import Flight Track", self.last_save_directory, name + " (*." + extension + ")")
             filename = filename[0] if isinstance(filename, tuple) and USE_PYQT5 else str(filename)
             if filename:
                 try:
@@ -329,15 +359,15 @@ class MSSMainWindow(QtWidgets.QMainWindow, ui.Ui_MSSMainWindow):
                     waypoints_model = ft.WaypointsTableModel(name=ft_name, waypoints=new_waypoints)
 
                     listitem = QFlightTrackListWidgetItem(waypoints_model, self.listFlightTracks)
-                    listitem.setFlags(QtCore.Qt.ItemIsEditable | QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsEnabled)
+                    listitem.setFlags(QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsEnabled)
 
                     self.listFlightTracks.setCurrentItem(listitem)
-                    self.setFlightTrackActive()
+                    self.activate_flight_track(listitem)
 
         setattr(self, full_name, types.MethodType(load_function_wrapper, self))
         action.triggered.connect(getattr(self, full_name))
 
-    def addExportFilter(self, name, extension, function):
+    def add_export_filter(self, name, extension, function):
         full_name = "actionExportFlightTrack" + clean_string(name)
         if hasattr(self, full_name):
             raise ValueError(u"'{}' has already been set!".format(full_name))
@@ -348,8 +378,9 @@ class MSSMainWindow(QtWidgets.QMainWindow, ui.Ui_MSSMainWindow):
         self.menuExport_Active_Flight_Track.addAction(action)
 
         def save_function_wrapper(self):
+            default_filename = os.path.join(self.last_save_directory, self.active_flight_track.name + "." + extension)
             filename = QtWidgets.QFileDialog.getSaveFileName(
-                self, "Export Flight Track", "", name + " (*." + extension + ")")
+                self, "Export Flight Track", default_filename, name + " (*." + extension + ")")
             filename = filename[0] if isinstance(filename, tuple) and USE_PYQT5 else str(filename)
             if filename:
                 try:
@@ -386,7 +417,7 @@ class MSSMainWindow(QtWidgets.QMainWindow, ui.Ui_MSSMainWindow):
         else:
             event.ignore()
 
-    def createNewView(self):
+    def create_new_view(self):
         """Method called when the user selects a new view to be opened. Creates
            a new instance of the view and adds a QActiveViewsListWidgetItem to
            the list of open views (self.listViews).
@@ -421,7 +452,7 @@ class MSSMainWindow(QtWidgets.QMainWindow, ui.Ui_MSSMainWindow):
             self.listViews.setCurrentItem(listitem)
             self.viewsChanged.emit()
 
-    def createNewTool(self):
+    def create_new_tool(self):
         """Method called when the user selects a new tool to be opened. Creates
            a new instance of the tool and adds a QActiveViewsListWidgetItem to
            the list of open tools (self.listTools).
@@ -435,10 +466,6 @@ class MSSMainWindow(QtWidgets.QMainWindow, ui.Ui_MSSMainWindow):
         elif self.sender() == self.actionTimeSeriesViewTrajectories:
             # Time series view.
             tool_window = timeseriesview.MSSTimeSeriesViewWindow()
-        elif self.sender() == self.actionLoopView:
-            # Loop view.
-            tool_window = loopview.MSSLoopWindow(
-                config_loader(dataset="loop_configuration", default=mss_default.loop_configuration))
 
         if tool_window is not None:
             # Make sure view window will be deleted after being closed, not
@@ -451,7 +478,7 @@ class MSSMainWindow(QtWidgets.QMainWindow, ui.Ui_MSSMainWindow):
             tool_window.viewCloses.connect(listitem.view_destroyed)
             self.viewsChanged.emit()
 
-    def activateSubWindow(self, item):
+    def activate_sub_window(self, item):
         """When the user clicks on one of the open view or tool windows, this
            window is brought to the front. This function implements the slot to
            activate a window if the user selects it in the list of views or
@@ -464,7 +491,7 @@ class MSSMainWindow(QtWidgets.QMainWindow, ui.Ui_MSSMainWindow):
 
     new_flight_track_counter = 0
 
-    def createNewFlightTrack(self, template=None, filename=None, activate=False):
+    def create_new_flight_track(self, template=None, filename=None):
         """Creates a new flight track model from a template. Adds a new entry to
            the list of flight tracks. Called when the user selects the 'new/open
            flight track' menu entries.
@@ -473,7 +500,6 @@ class MSSMainWindow(QtWidgets.QMainWindow, ui.Ui_MSSMainWindow):
         template -- copy the specified template to the new flight track (so that
                     it is not empty).
         filename -- if not None, load the flight track in the specified file.
-        activate -- set the new flight track to be the active flight track.
         """
         if template is None:
             template = []
@@ -498,18 +524,15 @@ class MSSMainWindow(QtWidgets.QMainWindow, ui.Ui_MSSMainWindow):
             # Make a copy of the template. Otherwise all new flight tracks would
             # use the same data structure in memory.
             template_copy = copy.deepcopy(template)
-            waypoints_model.insertRows(0, rows=len(template_copy),
-                                       waypoints=template_copy)
+            waypoints_model.insertRows(0, rows=len(template_copy), waypoints=template_copy)
         # Create a new list entry for the flight track. Make the item name
         # editable.
         listitem = QFlightTrackListWidgetItem(waypoints_model, self.listFlightTracks)
-        listitem.setFlags(QtCore.Qt.ItemIsEditable | QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsEnabled)
+        listitem.setFlags(QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsEnabled)
 
-        if activate:
-            self.listFlightTracks.setCurrentItem(listitem)
-            self.setFlightTrackActive()
+        self.activate_flight_track(listitem)
 
-    def openConfigFile(self):
+    def open_config_file(self):
         """
         Reads the config file
 
@@ -527,26 +550,36 @@ class MSSMainWindow(QtWidgets.QMainWindow, ui.Ui_MSSMainWindow):
             if filename:
                 self.listViews.clear()
                 self.listTools.clear()
-                self.removePlugins()
+                self.remove_plugins()
                 constants.CACHED_CONFIG_FILE = filename
-                self.addPlugins()
+                self.add_plugins()
 
-    def openFlightTrack(self):
+    def open_flight_track(self):
         """Slot for the 'Open Flight Track' menu entry. Opens a QFileDialog and
            passes the result to createNewFlightTrack().
         """
         filename = QtWidgets.QFileDialog.getOpenFileName(self,
-                                                         "Open Flight Track", "",
+                                                         "Open Flight Track", self.last_save_directory,
                                                          "Flight track XML (*.ftml)")
         filename = filename[0] if isinstance(filename, tuple) and USE_PYQT5 else str(filename)
         if filename:
-            if filename.endswith('.ftml'):
-                self.createNewFlightTrack(filename=filename, activate=True)
-            else:
-                QtWidgets.QMessageBox.warning(self, "Open flight track",
-                                              u"No supported file extension recognized!\n{:}".format(filename))
+            try:
+                if filename.endswith('.ftml'):
+                    self.create_new_flight_track(filename=filename)
+                else:
+                    QtWidgets.QMessageBox.warning(self, "Open flight track",
+                                                  u"No supported file extension recognized!\n{:}".format(filename))
 
-    def closeFlightTrack(self):
+            except (SyntaxError, OSError, IOError) as ex:
+                QtWidgets.QMessageBox.critical(
+                    self, self.tr("Problem while opening flight track FTML:"),
+                    self.tr(u"ERROR: {} {}".format(type(ex), ex)))
+
+    def activate_selected_flight_track(self):
+        item = self.listFlightTracks.currentItem()
+        self.activate_flight_track(item)
+
+    def close_selected_flight_track(self):
         """Slot to close the currently selected flight track. Flight tracks can
            only be closed if at least one other flight track remains open. The
            currently active flight track cannot be closed.
@@ -569,82 +602,78 @@ class MSSMainWindow(QtWidgets.QMainWindow, ui.Ui_MSSMainWindow):
             if ret == QtWidgets.QMessageBox.Yes:
                 self.listFlightTracks.takeItem(self.listFlightTracks.currentRow())
 
-    def saveFlightTrack(self):
+    def save_flight_track(self):
         """Slot for the 'Save Active Flight Track As' menu entry.
         """
-        filename = self.active_flight_track.getFilename()
+        filename = self.active_flight_track.get_filename()
         if filename and filename.endswith('.ftml'):
             sel = QtWidgets.QMessageBox.question(self, "Save flight track",
                                                  u"Saving flight track to '{:s}'. Continue?".format(filename),
                                                  QtWidgets.QMessageBox.Yes | QtWidgets.QMessageBox.No)
             if sel == QtWidgets.QMessageBox.Yes:
-                self.active_flight_track.saveToFTML(filename)
+                try:
+                    self.active_flight_track.save_to_ftml(filename)
+                except (OSError, IOError) as ex:
+                    QtWidgets.QMessageBox.critical(
+                        self, self.tr("Problem while saving flight track to FTML:"),
+                        self.tr(u"ERROR: {} {}".format(type(ex), ex)))
         else:
-            self.saveFlightTrackAs()
+            self.save_flight_track_as()
 
-    def saveFlightTrackAs(self):
+    def save_flight_track_as(self):
         """Slot for the 'Save Active Flight Track As' menu entry.
         """
-        default_filename = os.path.join(self.lastSaveDir, self.active_flight_track.name + ".ftml")
+        default_filename = os.path.join(self.last_save_directory, self.active_flight_track.name + ".ftml")
         filename = QtWidgets.QFileDialog.getSaveFileName(
             self, "Save Flight Track", default_filename, "Flight track XML (*.ftml)")
         filename = filename[0] if isinstance(filename, tuple) and USE_PYQT5 else str(filename)
         if filename:
-            self.lastSaveDir = os.path.dirname(filename)
+            self.last_save_directory = os.path.dirname(filename)
             if filename.endswith('.ftml'):
-                self.active_flight_track.saveToFTML(filename)
+                try:
+                    self.active_flight_track.save_to_ftml(filename)
+                except (OSError, IOError) as ex:
+                    QtWidgets.QMessageBox.critical(
+                        self, self.tr("Problem while saving flight track to FTML:"),
+                        self.tr(u"ERROR: {} {}".format(type(ex), ex)))
+                for idx in range(self.listFlightTracks.count()):
+                    if self.listFlightTracks.item(idx).flighttrack_model == self.active_flight_track:
+                        self.listFlightTracks.item(idx).setText(self.active_flight_track.name)
             else:
                 QtWidgets.QMessageBox.warning(self, "Save flight track",
                                               u"File extension is not '.ftml'!\n{:}".format(filename))
 
-    def setFlightTrackActive(self):
+    def activate_flight_track(self, item):
         """Set the currently selected flight track to be the active one, i.e.
            the one that is displayed in the views (only one flight track can be
            displayed at a time).
         """
-        item = self.listFlightTracks.currentItem()
         self.active_flight_track = item.flighttrack_model
-        self.lblActiveFlightTrack.setText(item.flighttrack_model.name)
         for i in range(self.listViews.count()):
             view_item = self.listViews.item(i)
             view_item.window.setFlightTrackModel(self.active_flight_track)
+        font = QtGui.QFont()
+        for i in range(self.listFlightTracks.count()):
+            self.listFlightTracks.item(i).setFont(font)
+        font.setBold(True)
+        item.setFont(font)
 
-    def flightTrackNameChanged(self, item):
-        """Slot to react to a name change of the flight tracks in the list
-           (i.e. when the user has edited a name). If the changed name
-           belongs to the currently active flight track, change its name
-           in the label that displays the active track.
-        """
-        if item.flighttrack_model == self.active_flight_track:
-            self.lblActiveFlightTrack.setText(item.text())
-        filename = item.flighttrack_model.filename \
-            if item.flighttrack_model.filename else ""
-        item.setToolTip(filename)
-
-    def showOnlineHelp(self):
+    def show_online_help(self):
         """Open Documentation in a browser"""
         QtGui.QDesktopServices.openUrl(
             QtCore.QUrl("http://mss.readthedocs.io/en/stable"))
 
-    def showAboutDlg(self):
+    def show_about_dialog(self):
         """Show the 'About MSUI' dialog to the user.
         """
         dlg = MSS_AboutDialog(parent=self)
         dlg.setModal(True)
         dlg.exec_()
 
-    def configureMenu(self):
+    def configure_menu(self):
         """
         disables menu entries which won't work, because libraries missing or not configured
         """
-        # loopview
-        try:
-            loop_configuration = config_loader(dataset="loop_configuration", default=mss_default.loop_configuration)
-            url = loop_configuration["ECMWF forecasts"]["url"]
-            requests.head(url)
-        except (KeyError, requests.exceptions.ConnectionError):
-            self.actionLoopView.setEnabled(False)
-
         # trajectory analyses
         if not HAVE_NAPPY:
             self.actionTrajectoryToolLagranto.setEnabled(False)
@@ -714,11 +743,15 @@ def main():
 
     setup_logging(args)
 
-    logging.info("Launching user interface...")
+    logging.info(u"MSS Version: %s", __version__)
+    logging.info(u"Python Version: %s", sys.version)
+    logging.info(u"Platform: %s (%s)", platform.platform(), platform.architecture())
+    logging.info(u"Launching user interface...")
+
     application = QtWidgets.QApplication(sys.argv)
     mainwindow = MSSMainWindow()
-    mainwindow.configureMenu()
-    mainwindow.createNewFlightTrack(activate=True)
+    mainwindow.configure_menu()
+    mainwindow.create_new_flight_track()
     mainwindow.show()
     sys.exit(application.exec_())
 
