@@ -127,15 +127,21 @@ class VS_GenericStyle(AbstractVerticalSectionStyle):
         if self.name[-2:] == "pl":
             self.data["air_pressure"] = np.empty_like(self.data[self.dataname])
             self.data["air_pressure"][:] = self.driver.vert_data[::-self.driver.vert_order, np.newaxis]
+            self.data_units["air_pressure"] = self.driver.vert_units
         elif self.name[-2:] == "tl":
             self.data["air_potential_temperature"] = np.empty_like(self.data[self.dataname])
             self.data["air_potential_temperature"][:] = self.driver.vert_data[::-self.driver.vert_order, np.newaxis]
 
+        if self.data_units["air_pressure"] not in ["Pa", "hPa"]:
+            raise ValueError("air_pressure neither hPa nor Pa: %s", self.data_units["air_pressure"])
+        if self.data_units["air_pressure"] == "hPa":
+            self.data["air_pressure"] *= 100
+
     def _plot_style(self):
         ax = self.ax
         curtain_cc = self.data[self.dataname] * self.unit_scale
         curtain_cc = np.ma.masked_invalid(curtain_cc)
-        curtain_p = self.data["air_pressure"] * 100
+        curtain_p = self.data["air_pressure"]
 
         numlevel = curtain_p.shape[0]
         numpoints = len(self.lats)
@@ -161,7 +167,7 @@ class VS_GenericStyle(AbstractVerticalSectionStyle):
         # Contour lines
         for cont_data, cont_levels, cont_colour, cont_label_colour, cont_style, cont_lw, pe in self.contours:
             if cont_levels is None:
-                pl_cont = ax.plot(self.lat_inds, self.data[cont_data].reshape(-1), "o", color="k", zorder=100)
+                pl_cont = ax.plot(self.lat_inds, self.data[cont_data].reshape(-1), "o", color=cont_colour, zorder=100)
                 plt.setp(pl_cont, path_effects=[patheffects.withStroke(linewidth=4, foreground="w")])
             else:
                 cs_pv = ax.contour(curtain_lat, curtain_p, self.data[cont_data], cont_levels,
@@ -236,43 +242,45 @@ def make_generic_class(name, entity, vert, add_data=None, add_contours=None,
 
 
 _ADD_DATA = {
-    "pl": [("pl", "ertel_potential_vorticity"), ("pl", "air_potential_temperature")],
+    "al": [("al", "ertel_potential_vorticity"), ("al", "air_pressure"), ("al", "air_potential_temperature")],
     "ml": [("ml", "ertel_potential_vorticity"), ("ml", "air_pressure"), ("ml", "air_potential_temperature")],
-    "tl": [("tl", "ertel_potential_vorticity"), ("tl", "air_pressure")]
+    "pl": [("pl", "ertel_potential_vorticity"), ("pl", "air_potential_temperature")],
+    "tl": [("tl", "ertel_potential_vorticity"), ("tl", "air_pressure")],
 }
 
-for vert in ["pl", "ml", "tl"]:
+for vert in ["al", "ml", "pl", "tl"]:
     for ent in Targets.get_targets():
         make_generic_class(
-            "VS_GenericStyle_{}_{}".format(vert.upper(), ent),
-            ent, vert, add_data=_ADD_DATA[vert])
+            "VS_GenericStyle_{}_{}".format(vert.upper(), ent), ent, vert,
+            add_data=_ADD_DATA[vert])
     make_generic_class(
         "VS_GenericStyle_{}_{}".format(vert.upper(), "ertel_potential_vorticity"),
-        "ertel_potential_vorticity", vert, add_data=_ADD_DATA[vert],
+        "ertel_potential_vorticity", vert,
+        add_data=_ADD_DATA[vert],
         fix_styles=[("ertel_potential_vorticity_nh", "northern hemisphere"),
                     ("ertel_potential_vorticity_sh", "southern hemisphere")])
     make_generic_class(
         "VS_GenericStyle_{}_{}".format(vert.upper(), "equivalent_latitude"),
-        "equivalent_latitude", vert, add_data=_ADD_DATA[vert],
+        "equivalent_latitude", vert,
+        add_data=_ADD_DATA[vert],
         fix_styles=[("equivalent_latitude_nh", "northern hemisphere"),
                     ("equivalent_latitude_sh", "southern hemisphere")])
+    make_generic_class(
+        "VS_GenericStyle_{}_{}".format(vert.upper(), "gravity_wave_temperature_perturbation"),
+        "air_temperature_residual", vert,
+        add_data=_ADD_DATA[vert] + [("sfc", "tropopause_air_pressure"),
+                                    ("sfc", "secondary_tropopause_air_pressure")],
+        add_contours=[("tropopause_air_pressure", None, "darkgrey", "darkgrey", "solid", 2, True),
+                      ("secondary_tropopause_air_pressure", None, "dimgrey", "dimgrey", "solid", 2, True)],
+        fix_styles=[("gravity_wave_temperature_perturbation", "")])
     make_generic_class(
         "VS_GenericStyle_{}_{}".format(vert.upper(), "square_of_brunt_vaisala_frequency_in_air"),
-        "square_of_brunt_vaisala_frequency_in_air", vert, add_data=_ADD_DATA[vert],
+        "square_of_brunt_vaisala_frequency_in_air", vert,
+        add_data=_ADD_DATA[vert] + [("sfc", "tropopause_air_pressure"),
+                                    ("sfc", "secondary_tropopause_air_pressure")],
+        add_contours=[("tropopause_air_pressure", None, "dimgrey", "dimgrey", "solid", 2, True),
+                      ("secondary_tropopause_air_pressure", None, "dimgrey", "dimgrey", "solid", 2, True)],
         fix_styles=[("square_of_brunt_vaisala_frequency_in_air", "")])
-vert = "ml"
-make_generic_class(
-    "VS_GenericStyle_{}_{}".format(vert.upper(), "gravity_wave_temperature_perturbation"),
-    "gravity_wave_temperature_perturbation", vert,
-    add_data=[("sfc", "tropopause_air_pressure"), ("ml", "air_pressure")],
-    add_contours=[("tropopause_air_pressure", None, "dimgrey", "dimgrey", "solid", 2, True)],
-    fix_styles=[("gravity_wave_temperature_perturbation", "")])
-make_generic_class(
-    "VS_GenericStyle_{}_{}".format(vert.upper(), "square_of_brunt_vaisala_frequency_in_air"),
-    "square_of_brunt_vaisala_frequency_in_air", vert,
-    add_data=[("sfc", "tropopause_air_pressure"), ("ml", "air_pressure")],
-    add_contours=[("tropopause_air_pressure", None, "dimgrey", "dimgrey", "solid", 2, True)],
-    fix_styles=[("square_of_brunt_vaisala_frequency_in_air", "")])
 
 vert = "pl"
 make_generic_class(
@@ -354,7 +362,6 @@ class VS_CloudsStyle_01(AbstractVerticalSectionStyle):
         # Contour spacing for temperature lines.
         delta_t = 2 if (np.log(self.p_bot) - np.log(self.p_top)) < 2.2 else 4
         delta_pt = 5 if (np.log(self.p_bot) - np.log(self.p_top)) < 2.2 else 10
-
         # Filled contour plot of cloud cover.
         # INFO on COLORMAPS:
         #    http://matplotlib.sourceforge.net/examples/pylab_examples/show_colormaps.html
@@ -712,7 +719,7 @@ class VS_VerticalVelocityStyle_01(AbstractVerticalSectionStyle):
     required_datafields = [
         ("ml", "air_pressure"),
         ("ml", "air_temperature"),
-        ("ml", "omega")]
+        ("ml", "lagrangian_tendency_of_air_pressure")]
 
     def _prepare_datafields(self):
         """Computes potential temperature from pressure and temperature if
@@ -724,7 +731,7 @@ class VS_VerticalVelocityStyle_01(AbstractVerticalSectionStyle):
                 thermolib.pot_temp(self.data['air_pressure'],
                                    self.data['air_temperature'])
         self.data["upward_wind"] = \
-            thermolib.omega_to_w(self.data["omega"],
+            thermolib.omega_to_w(self.data["lagrangian_tendency_of_air_pressure"],
                                  self.data['air_pressure'],
                                  self.data["air_temperature"])
 
@@ -1456,9 +1463,15 @@ class VS_MSSChemStyle(AbstractVerticalSectionStyle):
         self.title = self._title_tpl.format(modelname=self.driver.data_access._modelname)
         # for altitude level model data, when we don't have air_pressure information, we want to warn users that the
         # vertical section is only an approximation
-        if (self.name[-2:] == "al") and\
-                ("p" not in list(self.driver.data_access.build_filetree().values())[0].values()[0]):
-            self.title = self.title.replace(" al)", " al; WARNING: vert. distribution only approximate!)")
+        vert = self.name[-2:]
+        if vert != "pl":
+            # look for valid times including air_pressure
+            init_times = self.driver.data_access.get_init_times()
+            valid_times = self.driver.data_access.get_init_times.get_valid_times(
+                "air_pressure", vert, init_times[0])
+            if len(valid_times) == 0:
+                self.title = self.title.replace(
+                    " " + vert + ")", " " + vert + "; WARNING: vert. distribution only approximate!)")
 
     def _prepare_datafields(self):
         """Computes potential temperature from pressure and temperature if
@@ -1555,6 +1568,11 @@ def make_msschem_class(entity, nam, vert, units, scale, add_data=None,
             # at all (e.g., CAMS reg. Ensemble)
             # In those cases we derive air_pressure from the altitude alone, in the _prepare_datafields() method
             add_data = []
+        elif vert == 'pl':
+            # "pl" are pressure levels.  Here, the air_pressure information is implicitly contained in the vertical
+            # dimension coordinate, so we don't need to explicitly load it here.
+            add_data = []
+
         else:
             # all other layer types need to read air_pressure from the data
             add_data = [(vert, "air_pressure")]
