@@ -61,7 +61,7 @@ from chameleon import PageTemplateLoader
 try:
     import mss_wms_settings
 except ImportError as ex:
-    logging.warning(u"Couldn't import mss_wms_settings (ImportError:'{}'), creating dummy config.".format(ex))
+    logging.warning(u"Couldn't import mss_wms_settings (ImportError:'%s'), creating dummy config.", ex)
 
     class mss_wms_settings(object):
         base_dir = os.path.abspath(os.path.dirname(__file__))
@@ -81,17 +81,14 @@ except ImportError as ex:
         service_access_constraints = "This service is intended for research purposes only."
         register_horizontal_layers = []
         register_vertical_layers = []
-        nwpaccess = {
-            "ecmwf_NH_LL05": os.path.join(tempfile.gettempdir(), "mss/grid/ecmwf/netcdf")
-        }
+        data = {}
         enable_basic_http_authentication = False
         __file__ = None
 
-print(mss_wms_settings.__file__)
 try:
     import mss_wms_auth
 except ImportError as ex:
-    logging.warning(u"Couldn't import mss_wms_auth (ImportError:'{}'), creating dummy config.".format(ex))
+    logging.warning(u"Couldn't import mss_wms_auth (ImportError:'{%s), creating dummy config.", ex)
 
     class mss_wms_auth(object):
         allowed_users = [("mswms", "add_md5_digest_of_PASSWORD_here"),
@@ -99,7 +96,6 @@ except ImportError as ex:
         __file__ = None
 
 from mslib.mswms import mss_plot_driver
-
 from mslib.utils import CaseInsensitiveMultiDict
 
 # Logging the Standard Output, which will be added to the Apache Log Files
@@ -115,19 +111,33 @@ templates = PageTemplateLoader(mss_wms_settings.__dict__.get("xml_template_locat
 
 class WMSServer(object):
 
-    def __init__(self, data_access_dict):
-        self.hsec_layer_registry = {}
+    def __init__(self):
+        """
+        init method for wms server
+        """
+        data_access_dict = mss_wms_settings.data
+
+        for key in data_access_dict:
+            data_access_dict[key].setup()
+
         self.hsec_drivers = {}
         for key in data_access_dict:
             self.hsec_drivers[key] = mss_plot_driver.HorizontalSectionDriver(
                 data_access_dict[key])
 
-        self.vsec_layer_registry = {}
         self.vsec_drivers = {}
         for key in data_access_dict:
             self.vsec_drivers[key] = mss_plot_driver.VerticalSectionDriver(
                 data_access_dict[key])
 
+        self.hsec_layer_registry = {}
+        for layer, datasets in mss_wms_settings.register_horizontal_layers:
+            self.register_hsec_layer(datasets, layer)
+
+        self.vsec_layer_registry = {}
+        for layer, datasets in mss_wms_settings.register_vertical_layers:
+            self.register_vsec_layer(datasets, layer)
+
     def register_hsec_layer(self, datasets, layer_class):
         """Register horizontal section layer in internal dict of layers.
 
@@ -140,17 +150,19 @@ class WMSServer(object):
         # provided layer class for all datasets and register the layer
         # instances with the datasets.
         for dataset in datasets:
-            layer = layer_class()
-            logging.debug("registering horizontal section layer '{}' with "
-                          "dataset '{}'".format(layer.name, dataset))
+            try:
+                layer = layer_class(self.hsec_drivers[dataset])
+            except KeyError:
+                continue
+            logging.debug("registering horizontal section layer '%s' with "
+                          "dataset '%s'", layer.name, dataset)
             # Check if the current dataset has already been registered. If
             # not, check whether a suitable driver is available.
             if dataset not in self.hsec_layer_registry:
                 if dataset in self.hsec_drivers:
                     self.hsec_layer_registry[dataset] = {}
                 else:
-                    raise ValueError(u"dataset '{}' not available".format(dataset))
-            layer.set_driver(self.hsec_drivers[dataset])
+                    raise ValueError(u"dataset '%s' not available", dataset)
             self.hsec_layer_registry[dataset][layer.name] = layer
 
     def register_vsec_layer(self, datasets, layer_class):
@@ -162,16 +174,18 @@ class WMSServer(object):
         # provided layer class for all datasets and register the layer
         # instances with the datasets.
         for dataset in datasets:
-            layer = layer_class()
-            logging.debug(u"registering vertical section layer '{}' with dataset '{}'".format(layer.name, dataset))
+            try:
+                layer = layer_class(self.vsec_drivers[dataset])
+            except KeyError:
+                continue
+            logging.debug(u"registering vertical section layer '%s' with dataset '%s'", layer.name, dataset)
             # Check if the current dataset has already been registered. If
             # not, check whether a suitable driver is available.
             if dataset not in self.vsec_layer_registry:
                 if dataset in self.vsec_drivers:
                     self.vsec_layer_registry[dataset] = {}
                 else:
-                    raise ValueError(u"dataset '{}' not available".format(dataset))
-            layer.set_driver(self.vsec_drivers[dataset])
+                    raise ValueError(u"dataset '%s' not available", dataset)
             self.vsec_layer_registry[dataset][layer.name] = layer
 
     def is_service_exception(self, var):
@@ -198,8 +212,15 @@ class WMSServer(object):
         return [template(code=code, text=text), "text/xml"]
 
     def get_capabilities(self, server_url=None):
+        # ToDo find a more elegant method to do the same
+        # Preferable we don't want a seperate data_access module to be configured
+        data_access_dict = mss_wms_settings.data
+
+        for key in data_access_dict:
+            data_access_dict[key].setup()
+
         template = templates['get_capabilities.pt']
-        logging.debug("server-url '{0}'".format(server_url))
+        logging.debug("server-url '%s'", server_url)
 
         # Horizontal Layers
         hsec_layers = []
@@ -226,7 +247,6 @@ class WMSServer(object):
                 vsec_layers.append((dataset, layer))
 
         # return_format = 'text/xml'
-
         return_data = template(hsec_layers=hsec_layers, vsec_layers=vsec_layers, server_url=server_url,
                                service_name=mss_wms_settings.__dict__.get("service_name", "OGC:WMS"),
                                service_title=mss_wms_settings.__dict__.get("service_title",
@@ -269,7 +289,7 @@ class WMSServer(object):
 
         # Image size.
         figsize = float(query.get('WIDTH', 900)), float(query.get('HEIGHT', 600))
-        logging.debug("  requested image size = {:}x{:}".format(figsize[0], figsize[1]))
+        logging.debug("  requested image size = %sx%s", figsize[0], figsize[1])
 
         # Requested layers.
         layers = [layer for layer in query.get('LAYERS', '').strip().split(',') if layer]
@@ -278,12 +298,12 @@ class WMSServer(object):
             dataset, layer = layer.split(".")
         else:
             dataset = None
-        logging.debug(u"  requested dataset = '{}', layer = '{}'".format(dataset, layer))
+        logging.debug(u"  requested dataset = '%s', layer = '%s'", dataset, layer)
 
         # Requested style(s).
         styles = [style for style in query.get('STYLES', 'default').strip().split(',') if style]
         style = styles[0] if len(styles) > 0 else None
-        logging.debug(u"  requested style = '{}'".format(style))
+        logging.debug(u"  requested style = '%s'", style)
 
         # Forecast initialisation time.
         init_time = query.get('DIM_INIT_TIME')
@@ -291,10 +311,10 @@ class WMSServer(object):
             try:
                 init_time = datetime.strptime(init_time, "%Y-%m-%dT%H:%M:%SZ")
             except ValueError:
-                return self.service_exception(code="InvalidDimensionValue",
-                                              text="DIM_INIT_TIME has wrong format "
-                                              "(needs to be 2005-08-29T13:00:00Z)")
-        logging.debug(u"  requested initialisation time = '{}'".format(init_time))
+                return self.service_exception(
+                    code="InvalidDimensionValue",
+                    text="DIM_INIT_TIME has wrong format (needs to be 2005-08-29T13:00:00Z)")
+        logging.debug(u"  requested initialisation time = '%s'", init_time)
 
         # Forecast valid time.
         valid_time = query.get('TIME')
@@ -302,10 +322,10 @@ class WMSServer(object):
             try:
                 valid_time = datetime.strptime(valid_time, "%Y-%m-%dT%H:%M:%SZ")
             except ValueError:
-                return self.service_exception(code="InvalidDimensionValue",
-                                              text="TIME has wrong format "
-                                              "(needs to be 2005-08-29T13:00:00Z)")
-        logging.debug(u"  requested (valid) time = '{}'".format(valid_time))
+                return self.service_exception(
+                    code="InvalidDimensionValue",
+                    text="TIME has wrong format (needs to be 2005-08-29T13:00:00Z)")
+        logging.debug(u"  requested (valid) time = '%s'", valid_time)
 
         # Coordinate reference system.
         crs = query.get('SRS', 'EPSG:4326')
@@ -326,7 +346,7 @@ class WMSServer(object):
             logging.error(msg)
             return self.service_exception(code="InvalidSRS", text=msg)
         else:
-            logging.debug(u"  requested coordinate reference system = {}".format(crs))
+            logging.debug(u"  requested coordinate reference system = '%s'", crs)
 
         # Create a frameless figure (WMS) or one with title and legend
         # (MSS specific)? Default is WMS mode (frameless).
@@ -339,9 +359,9 @@ class WMSServer(object):
 
         # Return format (image/png, text/xml, etc.).
         return_format = query.get('FORMAT', 'IMAGE/PNG').lower()
-        logging.debug(u"  requested return format = '{}'".format(return_format))
+        logging.debug(u"  requested return format = '%s'", return_format)
         if return_format not in ["image/png", "text/xml"]:
-            msg = u"unsupported FORMAT: {}".format(return_format)
+            msg = u"unsupported FORMAT: '{}'".format(return_format)
             logging.error(msg)
             return self.service_exception(code="InvalidFORMAT", text=msg)
 
@@ -360,8 +380,7 @@ class WMSServer(object):
             # Check if the layer requires time information and if they are given.
             if self.hsec_layer_registry[dataset][layer].uses_time_dimensions():
                 if init_time is None:
-                    msg = "INIT_TIME not specified (use the "\
-                          "DIM_INIT_TIME keyword)"
+                    msg = "INIT_TIME not specified (use the DIM_INIT_TIME keyword)"
                     logging.error(msg)
                     return self.service_exception(code="MissingDimensionValue", text=msg)
                 if valid_time is None:
@@ -377,8 +396,7 @@ class WMSServer(object):
 
             # Bounding box.
             try:
-                bbox = [float(v) for v in query.get('BBOX',
-                                                    '-180,-90,180,90').split(',')]
+                bbox = [float(v) for v in query.get('BBOX', '-180,-90,180,90').split(',')]
             except ValueError:
                 msg = u"Invalid BBOX: {}".format(query.get("BBOX"))
                 logging.error(msg)
@@ -386,14 +404,14 @@ class WMSServer(object):
 
             # Vertical level, if applicable.
             level = query.get('ELEVATION')
-            level = int(level) if level else None
+            level = float(level) if level else None
             layer_datatypes = self.hsec_layer_registry[dataset][layer].required_datatypes()
-            if (("ml" in layer_datatypes) or ("pl" in layer_datatypes)) \
-                    and not level:
+            if any(_x in layer_datatypes for _x in ["pl", "al", "ml", "tl", "pv"]) and level is None:
                 # Use the default value.
                 level = -1
-            elif ("sfc" in layer_datatypes) and ("ml" not in layer_datatypes) \
-                    and ("pl" not in layer_datatypes) and level:
+            elif ("sfc" in layer_datatypes) and \
+                    all(_x not in layer_datatypes for _x in ["pl", "al", "ml", "tl", "pv"]) and \
+                    level is not None:
                 msg = u"ELEVATION argument not applicable for layer " \
                       u"'{}'. Please omit this argument.".format(layer)
                 logging.error(msg)
@@ -414,7 +432,7 @@ class WMSServer(object):
                                                 return_format=return_format)
                 image = plot_driver.plot()
             except (IOError, ValueError) as ex:
-                logging.error(u"ERROR: {}".format(ex))
+                logging.error(u"ERROR: %s %s", type(ex), ex)
                 msg = u"The data corresponding to your request " \
                       u"is not available. Please check the " \
                       u"times and/or levels you have specified." \
@@ -433,7 +451,7 @@ class WMSServer(object):
                 msg = u"Invalid PATH: {}".format(path)
                 logging.error(msg)
                 return self.service_exception(text=msg)
-            logging.debug(u"VSEC PATH: {}".format(path))
+            logging.debug(u"VSEC PATH: %s", path)
 
             # Check requested layers.
             if (dataset not in self.vsec_layer_registry) or \
@@ -478,7 +496,7 @@ class WMSServer(object):
                                                 return_format=return_format)
                 image = plot_driver.plot()
             except (IOError, ValueError) as ex:
-                logging.error(u"ERROR: {}".format(ex))
+                logging.error(u"ERROR: %s %s", type(ex), ex)
                 msg = u"The data corresponding to your request " \
                       u"is not available. Please check the " \
                       u"times and/or path you have specified." \
@@ -490,24 +508,14 @@ class WMSServer(object):
         return image, return_format
 
 
-app = WMSServer(mss_wms_settings.nwpaccess)
-
-for layer, datasets in mss_wms_settings.register_horizontal_layers:
-    app.register_hsec_layer(datasets, layer)
-
-for layer, datasets in mss_wms_settings.register_vertical_layers:
-    app.register_vsec_layer(datasets, layer)
-
-# Simple Caching Solutions
-# It maps the Requested URL to Tuble of ( data_format, response_body )
-cache = {}
+app = WMSServer()
 
 
 def application(environ, start_response):
     try:
         # Request info
         query = CaseInsensitiveMultiDict(paste.request.parse_dict_querystring(environ))
-        logging.debug(u"ENVIRON: {}".format(environ))
+        logging.debug(u"ENVIRON: %s", environ)
 
         # Processing
         request = str(query.get('request'))
@@ -517,25 +525,19 @@ def application(environ, start_response):
         url = paste.request.construct_url(environ)
         server_url = urllib.parse.urljoin(url, urllib.parse.urlparse(url).path)
 
-        if url in cache:
-            return_format, output = cache[url]
-        else:
-            if request.lower() == 'getcapabilities':
-                return_format = 'text/xml'
-                return_data = app.get_capabilities(server_url)
+        if request.lower() == 'getcapabilities':
+            return_format = 'text/xml'
+            return_data = app.get_capabilities(server_url)
+            output = str(return_data).encode('utf-8')
+        elif request.lower() in ['getmap', 'getvsec']:
+            return_data, return_format = app.produce_plot(environ, request)
+            # ToDo refactor
+            if not app.is_service_exception(return_data):
+                return_format = return_format.lower()  # MAYBE TO BE DELETED
+                output = return_data
+            else:
+                return_format = "text/xml"
                 output = str(return_data).encode('utf-8')
-            elif request.lower() in ['getmap', 'getvsec']:
-                return_data, return_format = app.produce_plot(environ, request)
-                # ToDo refactor
-                if not app.is_service_exception(return_data):
-                    return_format = return_format.lower()  # MAYBE TO BE DELETED
-                    output = return_data
-                else:
-                    return_format = "text/xml"
-                    output = str(return_data).encode('utf-8')
-
-                # Saving the result in a cache
-                cache[url] = (return_format, output)
 
         # Preparing the Response
         status = '200 OK'
